package com.jeeplus.quartz.task.pg;

import com.jeeplus.flowable.model.MyNoticeList;
import com.jeeplus.flowable.service.FlowTaskService;
import com.jeeplus.flowable.service.MyNoticeService;
import com.jeeplus.quartz.domain.Task;
import com.jeeplus.sys.service.dto.UserDTO;
import com.jeeplus.sys.utils.SpringContextHolder;
import com.jeeplus.sys.utils.StringUtils;
import com.jeeplus.sys.utils.UserUtils;
import com.jeeplus.test.program.configuration.projectList.domain.ProgramProjectListInfo;
import com.jeeplus.test.program.configuration.projectList.service.ProjectListService;
import org.quartz.DisallowConcurrentExecution;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 评估归档定时任务（每日凌晨20分进行处理，项目签发单完成后3天未归档，则调整为超期归档，并每天进行发送通知提醒。直到归档完成为止，并且项目终存在超期未归档时，禁止新增项目信息。）
 * @author: 徐滕
 * @version: 2023-06-08 09:36
 */
@DisallowConcurrentExecution
public class ProjectArchiveTask extends Task {

    private final ProjectListService pgProjectListService = SpringContextHolder.getBean(ProjectListService.class);
    private final FlowTaskService flowTaskService = SpringContextHolder.getBean(FlowTaskService.class);
    private final MyNoticeService myNoticeService = SpringContextHolder.getBean(MyNoticeService.class);

    @Override
    public void run() {
        List<ProgramProjectListInfo> projectList = pgProjectListService.getOverdueFilingProjectList();
        //遍历查询的项目，对所有项目的归档状态进行调整。添加超期归档状态值（over_archive_status：0：未超期；1：已超期），归档状态，若是0或者空值则调整为6。若为其他值则不进行调整
        for (ProgramProjectListInfo info : projectList) {
            //判定是否以发起归档申请（即归档状态是否为空或者0）
            String archiveStatus = "";
            if(StringUtils.isBlank(info.getArchiveStatus()) || "0".equals(info.getArchiveStatus())){
                archiveStatus = "6";
            }else{
                archiveStatus = info.getArchiveStatus();
            }
            //修改报告归档 归档状态信息
            pgProjectListService.updateArchiveStatus(info.getArchiveId(),archiveStatus,"1");
            //对数据进行发送通知
            String taskName = null;
            String titleStr = null;

            //如果当前时间大于过期时间
            //Date类的一个方法，如果info.getAdventDateDate()早于 new Date() 返回true，否则返回false
            if(info.getAgreeTime().before(new Date())){
                taskName = "评估-超期归档";
                titleStr = "项目【"+ info.getName()+"】归档已超期。超期时间："+dateToString(info.getAgreeTime()) +"。";
            }else{
                taskName = "评估-超期归档";
                titleStr = "项目【"+ info.getName()+"】即将归档超期。超期时间："+dateToString(info.getAgreeTime());
            }


            //计算两日期之间的天数
            int interval = Integer.parseInt(this.getInterval(info.getAgreeTime(), new Date()));
            if(interval > 0){
                titleStr = titleStr + "已超期：" + interval + "天。请尽快归还";
            }

            Set<String> noticeUserSet = new HashSet<String>();
            if(StringUtils.isNotBlank(info.getCreateBy())){
                UserDTO createUser = UserUtils.get(info.getCreateBy());
                if(null != createUser && StringUtils.isNotBlank(createUser.getId())){
                    noticeUserSet.add(createUser.getId());
                }
            }
            List<String> noticeUserList = new ArrayList<String>(noticeUserSet);
            if(noticeUserList.size()>0){
                MyNoticeList myNotice = new MyNoticeList();
                myNotice.setCreateBy("1");
                myNotice.setCreateDate(new Date());
                myNotice.setUpdateBy("1");
                myNotice.setUpdateDate(new Date());
                myNotice.setDelFlag(0);
                //根据procInsId获取taskId
                String currentTask = flowTaskService.getTaskIdByprocInstId(info.getProcInsId());
                myNotice.setTaskId(currentTask);
                myNotice.setDefId(info.getProcessDefinitionId());
                myNotice.setTitle(titleStr);
                myNotice.setTaskName(taskName);
                myNotice.setLink("结束");
                myNotice.setType("0");
                myNotice.setCreateUser("管理员");
                myNotice.setCreateTime(new Date());

                for (String noticeUserId : noticeUserList) {
                    myNotice.setNoticeId(noticeUserId);
                    myNotice.setNoticeName(UserUtils.get(noticeUserId).getName());
                    //根据taskName和通知人 查询重复数量
                    MyNoticeList repetitionCountBymyNotice = myNoticeService.getRepetitionCountBymyNoticeTitle(myNotice);
                    if(null == repetitionCountBymyNotice){
                        // 生成id
                        String id = UUID.randomUUID().toString().replace("-", "");
                        myNotice.setId(id);
                        myNotice.setRepetitionCount(0);
                        myNoticeService.insertMyNotice(myNotice);
                    }else{
                        myNotice.setId(repetitionCountBymyNotice.getId());
                        myNotice.setRepetitionCount(repetitionCountBymyNotice.getRepetitionCount()+1);
                        myNoticeService.updateMyNotice(myNotice);
                    }
                }
            }
        }

    }

    public static String dateToString(Date date) {
        SimpleDateFormat sformat = new SimpleDateFormat("yyyy-MM-dd");//日期格式
        return sformat.format(date);
    }


    /**
     * 计算两日期相差天数
     * @param beginDate
     * @param endDate
     * @return
     * @throws Exception
     */
    public String getInterval(Date beginDate, Date endDate){
        long day = 0;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        try {
            if(beginDate != null){
                String begin = sdf.format(beginDate);
                beginDate  = sdf.parse(begin);
            }
            if(endDate!= null){
                String end= sdf.format(endDate);
                endDate= sdf.parse(end);
            }
        } catch (Exception e){
            e.getMessage();
        }
        day = (endDate.getTime()-beginDate.getTime())/(24*60*60*1000);
        return String.valueOf(day);
    }
}
